#!/usr/bin/python3

import sys, os, re, urllib.parse
from html.parser import HTMLParser
from subprocess import Popen

def decode(string):
	return re.sub('&#[0-9]{5};', lambda x: chr(int(x.group(0)[2:-1])),
			re.sub('&nbsp;', ' ', string))

# 建立连接
import http.client
conn = http.client.HTTPConnection("www.google.cn")

# 下载搜索结果页面
if len(sys.argv) == 1:
	exit(0)

query = ' '.join(sys.argv[1:])
print ( "发送查询:", query )
query = urllib.parse.quote(query)
conn.request("GET", "http://www.google.cn/music/search?q="+query)
result = conn.getresponse()

# 分析页面, 得到歌曲列表
def get_attr(attrs, attr):
	for key, value in attrs:
		if key == attr:
			return value
	return None

class SongParser(HTMLParser):
	song_list = []
	def parse(self, htm):
		self.feed(htm.read().decode())
		return self.song_list

	def handle_starttag(self, tag, attrs):
		if get_attr(attrs, "class") == "SongItem BottomBorder":
			self.song_list.append( get_attr(attrs, "id")[3:] )

# 获取并分析下载页面
class SongInfoParser(HTMLParser):
	# song = [ name, singer, size, format ]
	tags = {
			"td-song-name":"name",
			"td-singer":"singer",
			"td-size":"size",
			"td-format":"format"
	}
	song = {}
	attr = None
	def parse(self, htm):
		self.feed(decode(htm.read().decode()))
		return self.song.copy()

	def handle_starttag(self, tag, attrs):
		if tag == "a":
			tag_href = get_attr(attrs, "href")
			if tag_href[0:7] == "/music/":
				self.song["address"] = "http://www.google.cn"+tag_href
		elif get_attr(attrs, "class") == "song-meta-data-table":
			self.attr = ''
		elif self.attr == '':
			tag_class = get_attr(attrs, "class")
			if tag_class in self.tags:
				self.attr = self.tags[tag_class]

	def handle_data(self, data):
		if self.attr:
			self.song[self.attr] = data

	def handle_endtag(self, tag):
		self.attr = ''

print ( "获取歌曲列表..." )
p = SongParser()
song_ids = p.parse( result )
songs = []
parser = SongInfoParser()

for i, id in enumerate(song_ids):
	conn.request("GET", "/music/top100/musicdownload?id="+id)
	response = conn.getresponse()
	song = parser.parse(response)
	name = song["name"]
	print ( i, '  '*(10-len(name))+name, song["singer"],
			song["format"], song["size"], sep='\t' )
	songs.append(song)


choice = input("选择要下载的歌曲['all' 或 '1, 2  3']: ")
choice = re.sub(',', ' ', choice).split()
if choice == []:
	print ( "什么也没下载" )
	exit(0)

if choice[0] == 'all':
	choice = range(len(songs))

# 下载歌曲
for i in choice:
	song = songs[ int(i) ]
	if isinstance(song, dict):
		print ( "下载", song["name"], "..." )
		Popen(['wget', '-c', '-P', song["singer"], song["address"]]).wait()

# 关闭连接
conn.close()

